using System;
using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio.TextManager.Interop;

using Irony.Parsing;

namespace Demo
{
    public class LineScanner : IScanner
    {
        private Parser parser;
        private const string parameterListStartChar = "(";
        private const string parameterListEndChar = ")";
        private const string parameterListNextChar = ",";
        private const string identifier = "identifier";



        public LineScanner(Grammar grammar)
        {
            this.parser = new Parser(grammar);
            this.parser.Context.Mode = ParseMode.VsLineScan;
        }

        public bool ScanTokenAndProvideInfoAboutIt(TokenInfo tokenInfo, ref int state)
        {
            // Reads each token in a source line and performs syntax coloring.  It will continue to
            // be called for the source until false is returned.

            Token token = parser.Scanner.VsReadToken(ref state);

            //new FilterToken(parser).FilteringToken(ref state);

            // !EOL and !EOF
            if (token != null && token.Terminal != Grammar.CurrentGrammar.Eof && token.Category != TokenCategory.Error)
            {
                string c = token.Text;
                tokenInfo.StartIndex = token.Location.Position;
                tokenInfo.EndIndex = tokenInfo.StartIndex + token.Length - 1;
                tokenInfo.Color = (Microsoft.VisualStudio.Package.TokenColor)token.EditorInfo.Color;
                tokenInfo.Type = (Microsoft.VisualStudio.Package.TokenType)token.EditorInfo.Type;


                if (token.Terminal.ToString() == identifier)
                {
                    tokenInfo.Trigger |= Microsoft.VisualStudio.Package.TokenTriggers.MemberSelect;
                }
                else if (c == parameterListStartChar)
                {
                    tokenInfo.Trigger |= Microsoft.VisualStudio.Package.TokenTriggers.ParameterStart;
                }
                else if (c == parameterListNextChar)
                {
                    tokenInfo.Trigger |= Microsoft.VisualStudio.Package.TokenTriggers.ParameterNext;
                }
                else if (c == parameterListEndChar)
                {
                    tokenInfo.Trigger |= Microsoft.VisualStudio.Package.TokenTriggers.ParameterEnd;
                }
                else if (token.KeyTerm != null)
                {
                    tokenInfo.Trigger =
                        (Microsoft.VisualStudio.Package.TokenTriggers)token.KeyTerm.EditorInfo.Triggers;
                }
                else
                {
                    tokenInfo.Trigger = (Microsoft.VisualStudio.Package.TokenTriggers)token.EditorInfo.Triggers;// Microsoft.VisualStudio.Package.TokenTriggers.Parameter;
                }


                return true;
            }

            return false;
        }

        public void SetSource(string source, int offset)
        {
            // Stores line of source to be used by ScanTokenAndProvideInfoAboutIt.
            parser.Scanner.VsSetSource(source, offset);
        }

    }

}

